// ***************************************************************************
// ***************************************************************************
// Rejeesh.Kutty@Analog.com (c) Analog Devices Inc.
// ***************************************************************************
// ***************************************************************************

module cf_adxl345 (

  spi_csn,
  spi_sclk,
  spi_sdo,
  spi_sdi,
  acl_int1,
  acl_int2,
  acl_status,
  debug_data,
  debug_trigger,

  up_rstn,
  up_clk,
  up_sel,
  up_rwn,
  up_addr,
  up_wdata,
  up_rdata,
  up_ack);

  parameter   C_SLV_DWIDTH = 32;
  parameter   C_NUM_REG = 32;
  parameter   ACL_IDLE      = 5'b00000;
  parameter   ACL_WAIT_INT  = 5'b00001;
  parameter   ACL_INIT_2B   = 5'b00010;
  parameter   ACL_READ_2B   = 5'b00011;
  parameter   ACL_UPDATE_2B = 5'b00100;
  parameter   ACL_WAIT_2B   = 5'b00101;
  parameter   ACL_INIT_30   = 5'b00110;
  parameter   ACL_READ_30   = 5'b00111;
  parameter   ACL_UPDATE_30 = 5'b01000;
  parameter   ACL_WAIT_30   = 5'b01001;
  parameter   ACL_INIT_39   = 5'b01010;
  parameter   ACL_READ_39   = 5'b01011;
  parameter   ACL_UPDATE_39 = 5'b01100;
  parameter   ACL_WAIT_39   = 5'b01101;
  parameter   ACL_INIT_32   = 5'b01110;
  parameter   ACL_READ_32   = 5'b01111;
  parameter   ACL_UPDATE_32 = 5'b10000;
  parameter   ACL_WAIT_32   = 5'b10001;

  output          spi_csn;
  output          spi_sclk;
  output          spi_sdo;
  input           spi_sdi;

  input           acl_int1;
  input           acl_int2;
  output  [ 3:0]  acl_status;

  output  [63:0]  debug_data;
  output  [ 7:0]  debug_trigger;

  input           up_rstn;
  input           up_clk;
  input           up_sel;
  input           up_rwn;
  input   [ 4:0]  up_addr;
  input   [31:0]  up_wdata;
  output  [31:0]  up_rdata;
  output          up_ack;

  reg     [ 3:0]  acl_int_status;
  reg     [ 4:0]  acl_state;
  reg             acl_spi_sclk_d1;
  reg             acl_spi_sdo_d1;
  reg             acl_spi_sclk_d2;
  reg             acl_spi_sdo_d2;
  reg             acl_spi_csn;
  reg             acl_spi_sclk;
  reg             acl_spi_sdo;
  reg     [10:0]  acl_spi_count;
  reg     [ 7:0]  acl_spi_data_out;
  reg     [47:0]  acl_spi_data_in;
  reg     [ 9:0]  acl_spi_delay;
  reg             cfg_spi_sclk_d1;
  reg             cfg_spi_sdo_d1;
  reg             cfg_spi_sclk_d2;
  reg             cfg_spi_sdo_d2;
  reg             cfg_spi_csn;
  reg             cfg_spi_sclk;
  reg             cfg_spi_sdo;
  reg             cfg_spi_update;
  reg     [ 8:0]  cfg_spi_count;
  reg     [15:0]  cfg_spi_data_out;
  reg     [ 7:0]  cfg_spi_data_in;
  reg             cfg_spi_init;
  reg             cfg_trigger_d;
  reg             acl_sw_trigger;
  reg             acl_int1_trigger;
  reg             acl_int2_trigger;
  reg     [ 7:0]  acl_act_tap_status;
  reg     [ 7:0]  acl_int_source;
  reg     [ 7:0]  acl_fifo_status;
  reg     [47:0]  acl_data;
  reg             acl_delta_x;
  reg             acl_delta_y;
  reg             acl_delta_z;
  reg             cfg_mode;
  reg             cfg_int_inv;
  reg             cfg_access;
  reg             cfg_trigger;
  reg     [ 3:0]  cfg_status;
  reg             cfg_rwn;
  reg     [ 5:0]  cfg_addr;
  reg     [ 7:0]  cfg_wdata;
  reg     [ 7:0]  cfg_rdata;
  reg             cfg_done;
  reg     [31:0]  up_rdata;
  reg             up_sel_d;
  reg             up_sel_2d;
  reg             up_ack;
  reg             spi_csn;
  reg             spi_sclk;
  reg             spi_sdo;
  reg     [ 3:0]  acl_status;
  reg             acl_int1_m1;
  reg             acl_int2_m1;
  reg             acl_int1_m2;
  reg             acl_int2_m2;

  wire            up_wr_s;
  wire            up_rd_s;
  wire            up_ack_s;

  assign debug_data = {acl_spi_delay[9:2],
    spi_csn,
    spi_sclk,
    spi_sdo,
    spi_sdi,
    acl_int_source,
    acl_state,
    acl_data[44:32],
    acl_data[28:16],
    acl_data[12:0]};

  assign debug_trigger = {acl_int_source[6:4],
    acl_int_source[2], acl_state[3:0]};

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      acl_int_status <= 4'd0;
    end else begin
      acl_int_status <= {(acl_int_source[2] | acl_int_source[6]),
        (acl_int_source[2] | acl_int_source[5]),
        (acl_delta_x | acl_delta_y), acl_delta_z};
    end
  end

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      acl_state <= ACL_IDLE;
    end else begin
      case (acl_state)
        ACL_IDLE: begin
          if (cfg_access == 1'b1) begin
            acl_state <= ACL_WAIT_INT;
          end else begin
            acl_state <= ACL_IDLE;
          end
        end
        ACL_WAIT_INT: begin
          if (cfg_access == 1'b0) begin
            acl_state <= ACL_IDLE;
          end else if ((acl_sw_trigger == 1) || (acl_int1_trigger == 1) ||
            (acl_int2_trigger == 1) || (cfg_mode == 1)) begin
            acl_state <= ACL_INIT_2B;
          end else begin
            acl_state <= ACL_WAIT_INT;
          end
        end
        ACL_INIT_2B: begin
          acl_state <= ACL_READ_2B;
        end
        ACL_READ_2B: begin
          if (acl_spi_count == 0) begin
            acl_state <= ACL_UPDATE_2B;
          end else begin
            acl_state <= ACL_READ_2B;
          end
        end
        ACL_UPDATE_2B: begin
         acl_state <= ACL_WAIT_2B;
        end
        ACL_WAIT_2B: begin
          if (acl_spi_delay == 0) begin
            acl_state <= ACL_INIT_30;
          end else begin
            acl_state <= ACL_WAIT_2B;
          end
        end
        ACL_INIT_30: begin
          acl_state <= ACL_READ_30;
        end
        ACL_READ_30: begin
          if (acl_spi_count == 0) begin
            acl_state <= ACL_UPDATE_30;
          end else begin
            acl_state <= ACL_READ_30;
          end
        end
        ACL_UPDATE_30: begin
          acl_state <= ACL_WAIT_30;
        end
        ACL_WAIT_30: begin
          if (acl_spi_delay == 0) begin
            acl_state <= ACL_INIT_39;
          end else begin
            acl_state <= ACL_WAIT_30;
          end
        end
        ACL_INIT_39: begin
          acl_state <= ACL_READ_39;
        end
        ACL_READ_39: begin
          if (acl_spi_count == 0) begin
            acl_state <= ACL_UPDATE_39;
          end else begin
            acl_state <= ACL_READ_39;
          end
        end
        ACL_UPDATE_39: begin
          acl_state <= ACL_WAIT_39;
        end
        ACL_WAIT_39: begin
          if (acl_spi_delay == 0) begin
            acl_state <= ACL_INIT_32;
          end else begin
            acl_state <= ACL_WAIT_39;
          end
        end
        ACL_INIT_32: begin
          acl_state <= ACL_READ_32;
        end
        ACL_READ_32: begin
          if (acl_spi_count == 0) begin
            acl_state <= ACL_UPDATE_32;
          end else begin
            acl_state <= ACL_READ_32;
          end
        end
        ACL_UPDATE_32: begin
          acl_state <= ACL_WAIT_32;
        end
        ACL_WAIT_32: begin
          if (acl_spi_delay == 0) begin
            acl_state <= ACL_WAIT_INT;
          end else begin
            acl_state <= ACL_WAIT_32;
          end
        end
        default: begin
          acl_state <= ACL_IDLE;
        end
      endcase
    end
  end

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      acl_spi_sclk_d1 <= 1'd0;
      acl_spi_sdo_d1 <= 1'd0;
      acl_spi_sclk_d2 <= 1'd0;
      acl_spi_sdo_d2 <= 1'd0;
      acl_spi_csn <= 1'd1;
      acl_spi_sclk <= 1'd0;
      acl_spi_sdo <= 1'd0;
      acl_spi_count <= 11'd0;
      acl_spi_data_out <= 8'd0;
      acl_spi_data_in <= 48'd0;
      acl_spi_delay <= 9'd0;
    end else begin
      acl_spi_sclk_d1 <= ~acl_spi_count[10] | acl_spi_count[3];
      acl_spi_sdo_d1 <= acl_spi_data_out[7];
      acl_spi_sclk_d2 <= acl_spi_sclk_d1;
      acl_spi_sdo_d2 <= acl_spi_sdo_d1;
      acl_spi_csn <= ~acl_spi_count[10];
      acl_spi_sclk <= acl_spi_sclk_d2;
      acl_spi_sdo <= acl_spi_sdo_d2;
      if (acl_spi_count[10] == 1'b1) begin
        acl_spi_count <= acl_spi_count + 1;
        if (acl_spi_count[3:0] == 4'b1111) begin
          acl_spi_data_out <= {acl_spi_data_out[6:0], 1'b0};
        end
        if ((acl_spi_count[9:7] >= 3'b010) &&
          (acl_spi_count[3:0] == 4'b1111)) begin
          acl_spi_data_in <= {acl_spi_data_in[46:0], spi_sdi};
        end
      end else begin
        case (acl_state)
          ACL_INIT_2B: begin
            acl_spi_count <= 11'h700;
            acl_spi_data_out <= {1'b1, 1'b0, 6'h2b};
            acl_spi_data_in <= 'd0;
          end
          ACL_INIT_30: begin
            acl_spi_count <= 11'h700;
            acl_spi_data_out <= {1'b1, 1'b0, 6'h30};
            acl_spi_data_in <= 'd0;
          end
          ACL_INIT_39: begin
            acl_spi_count <= 11'h700;
            acl_spi_data_out <= {1'b1, 1'b0, 6'h39};
            acl_spi_data_in <= 'd0;
          end
          ACL_INIT_32: begin
            acl_spi_count <= 11'h480;
            acl_spi_data_out <= {1'b1, 1'b1, 6'h32};
            acl_spi_data_in <= 'd0;
          end
        endcase
      end
      if (acl_spi_delay[9] == 1) begin
        acl_spi_delay <= acl_spi_delay + 1;
      end else if ((acl_state == ACL_UPDATE_2B) ||
        (acl_state == ACL_UPDATE_30) ||
        (acl_state == ACL_UPDATE_39)) begin
        acl_spi_delay <= 10'h3c0;
      end else if (acl_state == ACL_UPDATE_32) begin
        acl_spi_delay <= 10'h200;
      end
    end
  end

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      cfg_spi_sclk_d1 <= 1'd0;
      cfg_spi_sdo_d1 <= 1'd0;
      cfg_spi_sclk_d2 <= 1'd0;
      cfg_spi_sdo_d2 <= 1'd0;
      cfg_spi_csn <= 1'd1;
      cfg_spi_sclk <= 1'd0;
      cfg_spi_sdo <= 1'd0;
      cfg_spi_update <= 1'd0;
      cfg_spi_count <= 9'd0;
      cfg_spi_data_out <= 16'd0;
      cfg_spi_data_in <= 8'd0;
    end else begin
      cfg_spi_sclk_d1 <= ~cfg_spi_count[8] | cfg_spi_count[3];
      cfg_spi_sdo_d1 <= cfg_spi_data_out[15];
      cfg_spi_sclk_d2 <= cfg_spi_sclk_d1;
      cfg_spi_sdo_d2 <= cfg_spi_sdo_d1;
      cfg_spi_csn <= ~cfg_spi_count[8];
      cfg_spi_sclk <= cfg_spi_sclk_d2;
      cfg_spi_sdo <= cfg_spi_sdo_d2;
      cfg_spi_update <= (cfg_spi_count == 9'h1ff) ? 1'b1 : 1'b0;
      if (cfg_spi_count[8] == 1'b1) begin
        cfg_spi_count <= cfg_spi_count + 1;
        if (cfg_spi_count[3:0] == 4'b1111) begin
          cfg_spi_data_out <= {cfg_spi_data_out[14:0], 1'b0};
          if ((cfg_rwn == 1'b1) && (cfg_spi_count[7] == 1'b1)) begin
            cfg_spi_data_in <= {cfg_spi_data_in[6:0], spi_sdi};
          end
        end
      end else begin
        if ((cfg_spi_init == 1) && (acl_state == 0)) begin
          cfg_spi_count <= 9'h100;
          if (cfg_rwn == 1'b1) begin
            cfg_spi_data_out <= {cfg_rwn, 1'b0, cfg_addr, 8'd0};
          end else begin
            cfg_spi_data_out <= {cfg_rwn, 1'b0, cfg_addr, cfg_wdata};
          end
          cfg_spi_data_in <= 8'd0;
        end
      end
    end
  end

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      cfg_spi_init <= 1'd0;
      cfg_trigger_d <= 1'd0;
      acl_sw_trigger <= 1'd0;
      acl_int1_trigger <= 1'd0;
      acl_int2_trigger <= 1'd0;
    end else begin
      cfg_spi_init <= (up_addr == 5'h01) ? up_wr_s : 1'b0;
      cfg_trigger_d <= cfg_trigger;
      acl_sw_trigger <= cfg_trigger & ~cfg_trigger_d;
      acl_int1_trigger <= acl_int1_m2 ^ cfg_int_inv;
      acl_int2_trigger <= acl_int2_m2 ^ cfg_int_inv;
    end
  end

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      acl_act_tap_status <= 8'd0;
      acl_int_source <= 8'd0;
      acl_fifo_status <= 8'd0;
      acl_data <= 48'd0;
      acl_delta_x <= 48'd0;
      acl_delta_y <= 48'd0;
      acl_delta_z <= 48'd0;
    end else begin
      if (acl_state == ACL_UPDATE_2B) begin
        acl_act_tap_status <= acl_spi_data_in[7:0];
      end else if (acl_state == ACL_IDLE) begin
        acl_act_tap_status <= 'd0;
      end
      if (acl_state == ACL_UPDATE_30) begin
        acl_int_source <= acl_int_source | acl_spi_data_in[7:0];
      end else if ((up_addr == 5'h06) && (up_wr_s == 1'b1)) begin
        acl_int_source <= 'd0;
      end
      if (acl_state == ACL_UPDATE_39) begin
        acl_fifo_status <= acl_spi_data_in[7:0];
      end else if (acl_state == ACL_IDLE) begin
        acl_fifo_status <= 'd0;
      end
      if (acl_state == ACL_UPDATE_32) begin
        acl_data <= {acl_spi_data_in[39:32], acl_spi_data_in[47:40],
          acl_spi_data_in[23:16], acl_spi_data_in[31:24],
          acl_spi_data_in[7:0], acl_spi_data_in[15:8]};
        acl_delta_x <= acl_spi_data_in[32];
        acl_delta_y <= acl_spi_data_in[16];
        acl_delta_z <= acl_spi_data_in[0];
      end else if (acl_state == ACL_IDLE) begin
        acl_data <= 'd0;
        acl_delta_x <= 1'b0;
        acl_delta_y <= 1'b0;
        acl_delta_z <= 1'b0;
      end
    end
  end

  assign up_wr_s = up_sel & ~up_rwn;
  assign up_rd_s = up_sel & up_rwn;
  assign up_ack_s = up_sel_d & ~up_sel_2d;

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      cfg_mode <= 1'd0;
      cfg_int_inv <= 1'd0;
      cfg_access <= 1'd0;
      cfg_trigger <= 1'd0;
      cfg_status <= 4'd3;
      cfg_rwn <= 1'd0;
      cfg_addr <= 6'd0;
      cfg_wdata <= 8'd0;
      cfg_rdata <= 8'd0;
      cfg_done <= 1'd0;
    end else begin
      if ((up_addr == 5'h00) && (up_wr_s == 1'b1)) begin
        cfg_mode <= up_wdata[7];
        cfg_int_inv <= up_wdata[6];
        cfg_access <= up_wdata[5];
        cfg_trigger <= up_wdata[4];
        cfg_status <= up_wdata[3:0];
      end
      if ((up_addr == 5'h01) && (up_wr_s == 1'b1)) begin
        cfg_rwn <= up_wdata[31];
        cfg_addr <= up_wdata[29:24];
        cfg_wdata <= up_wdata[23:16];
      end
      if ((cfg_spi_update == 1) && (cfg_rwn == 1)) begin
        cfg_rdata <= cfg_spi_data_in;
      end else if ((cfg_spi_init == 1) && (cfg_rwn == 1)) begin
        cfg_rdata <= 'd0;
      end
      if (cfg_spi_update == 1) begin
        cfg_done <= 1'b1;
      end else if (cfg_spi_init == 1) begin
        cfg_done <= 1'b0;
      end
    end
  end

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      up_rdata <= 'd0;
      up_sel_d <= 'd0;
      up_sel_2d <= 'd0;
      up_ack <= 'd0;
    end else begin
      case (up_addr)
        5'h00: up_rdata <= {24'd0, cfg_mode, cfg_int_inv, cfg_access,
          cfg_trigger, cfg_status};
        5'h01: up_rdata <= {cfg_rwn, 1'd0, cfg_addr, cfg_wdata,
          7'd0, cfg_done, cfg_rdata};
        5'h02: up_rdata <= {16'd0, acl_data[15:0]};
        5'h03: up_rdata <= {16'd0, acl_data[31:16]};
        5'h04: up_rdata <= {16'd0, acl_data[47:32]};
        5'h05: up_rdata <= {24'd0, acl_act_tap_status};
        5'h06: up_rdata <= {24'd0, acl_int_source};
        5'h07: up_rdata <= {24'd0, acl_fifo_status};
        5'h08: up_rdata <= {28'd0, acl_state};
        default: up_rdata <= 0;
      endcase
      up_sel_d <= up_sel;
      up_sel_2d <= up_sel_d;
      up_ack <= up_ack_s;
    end
  end

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      spi_csn <= 'd1;
      spi_sclk <= 'd0;
      spi_sdo <= 'd0;
      acl_status <= 'd0;
    end else begin
      if (acl_state == ACL_IDLE) begin
        spi_csn <= cfg_spi_csn;
        spi_sclk <= cfg_spi_sclk;
        spi_sdo <= cfg_spi_sdo;
        acl_status <= cfg_status;
      end else begin
        spi_csn <= acl_spi_csn;
        spi_sclk <= acl_spi_sclk;
        spi_sdo <= acl_spi_sdo;
        acl_status <= acl_int_status;
      end
    end
  end

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      acl_int1_m1 <= 'd0;
      acl_int2_m1 <= 'd0;
      acl_int1_m2 <= 'd0;
      acl_int2_m2 <= 'd0;
    end else begin
      acl_int1_m1 <= acl_int1;
      acl_int2_m1 <= acl_int2;
      acl_int1_m2 <= acl_int1_m1;
      acl_int2_m2 <= acl_int2_m1;
    end
  end

endmodule

// ***************************************************************************
// ***************************************************************************
