// ***************************************************************************
// ***************************************************************************
// Rejeesh.Kutty@Analog.com (c) Analog Devices Inc.
// ***************************************************************************
// ***************************************************************************

#include <stdio.h>
#include "platform.h"
#include "xbasic_types.h"
#include "xstatus.h"
#include "xil_io.h"
#include "xparameters.h"
#include "xuartlite_l.h"

#define ACL_BASEADDR XPAR_CF_ADXL345_CORE_0_BASEADDR

extern char inbyte(void);

u32 acl_exit(void) {

  u32 acl_quit = 0;

  while (!XUartLite_IsReceiveEmpty(XPAR_RS232_BASEADDR)) {
    if (XUartLite_RecvByte(XPAR_RS232_BASEADDR) == 'q') {
      acl_quit = 1;
      break;
    }
  }

  return(acl_quit);
}

void acl_delay_ms(u32 ms_count) {

  u32 count;

  for (count = 0; count < ((ms_count * 66000) + 1); count++) {
    asm("nop");
  }
}

void acl_spi_write(u32 addr, u32 data) {

  u32 data_w = 0;

  data_w = (((addr << 8) | data) << 16) & 0xffff0000;

  Xil_Out32((ACL_BASEADDR + 0x004), data_w);

  while ((Xil_In32(ACL_BASEADDR + 0x004) & 0x00000100) == 0) {
    acl_delay_ms(1);
  }
}

u32 acl_spi_read(u32 addr) {

  u32 data_r = 0;
  u32 data_w = 0;

  data_w = ((addr << 24) | 0x80000000) & 0xffff0000;

  Xil_Out32((ACL_BASEADDR + 0x004), data_w);

  while ((Xil_In32(ACL_BASEADDR + 0x004) & 0x00000100) == 0) {
    acl_delay_ms(1);
  }

  data_r = Xil_In32(ACL_BASEADDR + 0x04) & 0xff;
  return(data_r);
}

// ***************************************************************************
// ***************************************************************************

int main() {

  u32 acl_datax;
  u32 acl_datay;
  u32 acl_dataz;
  u32 acl_datai;
  u32 acl_stap;
  u32 acl_dtap;
  u32 acl_ffall;

  int acl_datax_int;
  int acl_datay_int;
  int acl_dataz_int;

  init_platform();

  xil_printf("\n\rreseting device, please wait\n\r");
  acl_spi_write(0x01, 0x52); // soft reset
  acl_delay_ms(2000);

  acl_spi_write(0x2c, 0x0a); // data update rate (100Hz)
  acl_spi_write(0x31, 0x0b); // data format (measure upto 16g), full res = 1
  acl_spi_write(0x2e, 0x74); // enable single, double tap and free fall interrupts
  acl_spi_write(0x1d, 0x50); // tap threshold
  acl_spi_write(0x21, 0x0d); // tap duration
  acl_spi_write(0x22, 0x50); // tap latency
  acl_spi_write(0x23, 0xf0); // tap window
  acl_spi_write(0x2a, 0x07); // enable all tap axes
  acl_spi_write(0x24, 0x08); // activity threshold
  acl_spi_write(0x27, 0x70); // activity axes
  acl_spi_write(0x28, 0x08); // free fall threshold
  acl_spi_write(0x29, 0x0a); // free fall window
  acl_spi_write(0x2d, 0x08); // enable measurement

  xil_printf("\n\rreading parameters\n\r");
  xil_printf("addr(0x%02x): data(0x%02x)\n\r", 0x2c, acl_spi_read(0x2c));
  xil_printf("addr(0x%02x): data(0x%02x)\n\r", 0x31, acl_spi_read(0x31));
  xil_printf("addr(0x%02x): data(0x%02x)\n\r", 0x2e, acl_spi_read(0x2e));
  xil_printf("addr(0x%02x): data(0x%02x)\n\r", 0x1d, acl_spi_read(0x1d));
  xil_printf("addr(0x%02x): data(0x%02x)\n\r", 0x21, acl_spi_read(0x21));
  xil_printf("addr(0x%02x): data(0x%02x)\n\r", 0x22, acl_spi_read(0x22));
  xil_printf("addr(0x%02x): data(0x%02x)\n\r", 0x23, acl_spi_read(0x23));
  xil_printf("addr(0x%02x): data(0x%02x)\n\r", 0x2a, acl_spi_read(0x2a));
  xil_printf("addr(0x%02x): data(0x%02x)\n\r", 0x24, acl_spi_read(0x24));
  xil_printf("addr(0x%02x): data(0x%02x)\n\r", 0x27, acl_spi_read(0x27));
  xil_printf("addr(0x%02x): data(0x%02x)\n\r", 0x28, acl_spi_read(0x28));
  xil_printf("addr(0x%02x): data(0x%02x)\n\r", 0x29, acl_spi_read(0x29));
  xil_printf("addr(0x%02x): data(0x%02x)\n\r", 0x2d, acl_spi_read(0x2d));

  xil_printf("\n\rstarting measurement, press 'q' to exit\n\r");
  Xil_Out32((ACL_BASEADDR + 0x000), 0xa0);

  while (acl_exit() == 0) {

    acl_delay_ms(100);
    acl_datai = Xil_In32(ACL_BASEADDR + 0x18);
    Xil_Out32((ACL_BASEADDR + 0x018), 0xff);

    acl_datax = Xil_In32(ACL_BASEADDR + 0x10);
    acl_datay = Xil_In32(ACL_BASEADDR + 0x0c);
    acl_dataz = Xil_In32(ACL_BASEADDR + 0x08);

    if ((acl_datax & 0x1000) == 0x1000) {
      acl_datax = acl_datax | 0xfffff000;
    }
    if ((acl_datay & 0x1000) == 0x1000) {
      acl_datay = acl_datay | 0xfffff000;
    }
    if ((acl_dataz & 0x1000) == 0x1000) {
      acl_dataz = acl_dataz | 0xfffff000;
    }

    acl_datax_int = (int) acl_datax;
    acl_datay_int = (int) acl_datay;
    acl_dataz_int = (int) acl_dataz;

    acl_datax_int = acl_datax_int/2;
    acl_datay_int = acl_datay_int/2;
    acl_dataz_int = acl_dataz_int/2;

    acl_stap = ((acl_datai & 0x40) == 0) ? 0 : 1;
    acl_dtap = ((acl_datai & 0x20) == 0) ? 0 : 1;
    acl_ffall = ((acl_datai & 0x04) == 0) ? 0 : 1;

    xil_printf("x(%5d), y(%5d), z(%5d) st(%d) dt(%d) ff(%d) i(0x%04x)\n\r",
      acl_datax_int, acl_datay_int, acl_dataz_int,
      acl_stap, acl_dtap, acl_ffall, acl_datai);
  }

  xil_printf("exiting\r");
  cleanup_platform();

  return 0;
}


// ***************************************************************************
// ***************************************************************************

